#!/usr/bin/perl
#

# Reverse DNS generator. For use with the NorthStar IP Tracking System 
# (http://www.brownkid.net/NorthStar)
#
# This program searches through the northstar networks table and retrieves 
# all addresses with a masklen of 32. It then creates a standard bind 
# reverse DNS file from that data.
#
# Project Homepage: http://northstar.reachone.net
#
# Creation date: 04/26/02 by Adam "Tauvix" Debus (adebus@reachone.com)
#
# See CHANGELOG for updates
#
# Change the information below to match your information. 

# Set your DB Info here

my $dbhost = 'localhost';
my $dbase = 'northstar';
my $dbuser = 'northstar';
my $dbpass = 'password';

# Set where you want the files to be put, include trailing slash.
my $fileloc = '/var/named/arpa/';

# Set where you want the forward DNS file (specify domain below) to be put.
# Include trailing slash.
my $fowardfile = '/var/named/';

# Set your name server addresses here.
my $ns1 = 'ns1.domain.com';
my $ns2 = 'ns2.domain.com';

# Set your domain here.
my $domain = 'domain.com';

# Set any forward DNS records you want here.
# Format: "name", "type", "address"

my @forward = ( [ "www", "A", "0.0.0.0" ],
		[ "mail", "CNAME", "mail.domain.com" ]
);

# Set your primary domain address here, usually the same as your web server
my $domaddr = "0.0.0.0";

# Set your MX records here
# Format: "Priority", "Server"
my @mx = ( [ "0", "mail" ],
	   [ "10", "0.0.0.0" ]
);

# Shouldn't have to change anything below this line.

Main();


# Functions
sub Main {

	use DBI;
	$db = DBI->connect("DBI:mysql:$dbase:$dbhost", $dbuser, $dbpass);

	my @slash24s = Populate24s($db);

	CreateFiles(@slash24s);

	$db->disconnect();
}

sub GetLocalTime {

	my ($sec,$min,$hour,$mday,$mon,$year,$wday,$yday,$isdst) = localtime();
	$year = $year+1900;
	$mon = $mon+1;
	my $today = 0;
	if ($mon < 10) {
		$today = $year."0".$mon.$mday."01";
	}
	else {
		$today = $year.$mon.$mday."01";
	}
	return $today;
}

sub FileHeader {

	my($ns1, $ns2, $domain, $serial) = @_;

	my $header = <<"END";
@	in	soa	$ns1.	hostmaster.$domain. (
		$serial	; Serial
		7200		; Refresh after 2 hours
		3600		; Retry after 1 hour
		10800		; Expire after 1 week
		86400	)	; Minimum TTL of 1 day

; Auto Generated by gen-reverse.pl
; Please make all updates in the NorthStar IP Tracking System.

	in	ns	$ns1.
	in	ns	$ns2.
END

	return $header;
}

sub ForwardHeader {

	my $serial = shift(@_);

	my $header = <<"END";
@	in	soa	$ns1.	hostmaster.$domain. (
		$serial	; Serial
		7200		; Refresh after 2 hours
		3600		; Retry after 1 hour
		10800		; Expire after 1 week
		86400	)	; Minimum TTL of 1 day

; Auto Generated by gen-reverse.pl
; Please make all updates in the NorthStar IP Tracking System.
; For Static Forward Mappings, please edit gen-reverse.pl

	in	ns	$ns1.
	in	ns	$ns2.

@		A	$domaddr
END

	return $header;
}

sub CreateForward {

	my (@forwardlist, $counter, $header);

	my $query = $db->prepare('select name,label from networks where masklen="32" and name like "%$domain" order by network');

	$query->execute();

	my $results_count = $query->rows();

	my $today = GetLocalTime();

	open(FILE,">$fowardfile"."$domain");	

	print FILE ForwardHeader($today);

	print FILE "\n";

	for $i ( 0 .. $#mx ) {
		print FILE "@\t\tMX\t$mx[$i][0]\t$mx[$i][1]\n";
	}

	print FILE "\n";

	for $i (0 .. $#forward ) {
		if ($forward[$i][1] eq 'A') {
			print FILE "$forward[$i][0]\t$forward[$i][1]\t$forward[$i][2]\n";
		} elsif ($forward[$i][1] eq 'CNAME') {
			print FILE "$forward[$i][0]\t$forward[$i][1]\t$forward[$i][2].\n";
		}
	}

	print FILE "\n";

	while ( ($name, $label) = $query->fetchrow_array ) {
		my ($address, $mask) = split /\//,$label;

		my ($name2, $domain, $tld) = split /\./,$name;

		print FILE "$name2\t\tA\t$address\n";
	}

	close FILE;
}

sub Populate24s {

	my $db = shift(@_);
	my ($label, $masklen, $counter, $counter2, @array);

	my $query = $db->prepare('select label,masklen from networks where masklen < 25 and masklen > 0 order by network,masklen');

	$query->execute();

	my $results_count = $query->rows();

	$counter = 0;
	$counter2 = 0;
	while ( ($label, $masklen) = $query->fetchrow_array ) {

		my ($network, $mask) = split /\//,$label;

		my $no24s;

		if ($counter eq 0) {
			$no24s = TranslateCIDR($masklen);
			for ($counter2 = 0; $counter2 < $no24s; $counter2++) {
				my ($oc1, $oc2, $oc3, $oc4) = split /\./,$network;
				my $newnet = "$oc1.$oc2.".($oc3 + $counter2).".$oc4";
				@array[$counter] = $newnet;
				$counter++;
			}
		} else {
			my $found = 0;
			for ($counter2 = 0; $counter2 < $counter; $counter2++) {
				if ($found == 0) {
					if ($network eq @array[$counter2]) {
						$found =1;
					}
				}
			}
			if ($found == 0) {
				$no24s = TranslateCIDR($masklen);
				for ($counter2 = 0; $counter2 < $no24s; $counter2++) {
					my ($oc1, $oc2, $oc3, $oc4) = split /\./,$network;
					my $newnet = "$oc1.$oc2.".($oc3 + $counter2).".$oc4";
					@array[$counter] = $newnet;
					$counter++;
				}
			}
		}
	}

	return @array;
}

sub TranslateCIDR {

	my $masklen = shift(@_);
	my $no24s = 0;

	if ($masklen eq 24) {
		$no24s = 1;
	} elsif ($masklen eq 23) {
		$no24s = 2;
	} elsif ($masklen eq 22) {
		$no24s = 4;
	} elsif ($masklen eq 21) {
		$no24s = 8;
	} elsif ($masklen eq 20) {
		$no24s = 16;
	} elsif ($masklen eq 19) {
		$no24s = 32;
	} elsif ($masklen eq 18) {
		$no24s = 64;
	} elsif ($masklen eq 17) {
		$no24s = 128;
	} elsif ($masklen eq 16) {
		$no24s = 256;
	} elsif ($masklen eq 15) {
		$no24s = 512;
	} elsif ($masklen eq 14) {
		$no24s = 1024;
	} elsif ($masklen eq 13) {
		$no24s = 2048;
	} elsif ($masklen eq 12) {
		$no24s = 4096;
	} elsif ($masklen eq 11) {
		$no24s = 8192;
	} elsif ($masklen eq 10) {
		$no24s = 16384;
	} elsif ($masklen eq 9) {
		$no24s = 32768;
	} elsif ($masklen eq 8) {
		$no24s = 65536;
	} elsif ($masklen eq 7) {
		$no24s = 131072;
	} elsif ($masklen eq 6) {
		$no24s = 262144;
	} elsif ($masklen eq 5) {
		$no24s = 524288;
	} elsif ($masklen eq 4) {
		$no24s = 1048576;
	} elsif ($masklen eq 3) {
		$no24s = 2097152;
	} elsif ($masklen eq 2) {
		$no24s = 4194304;
	} elsif ($masklen eq 1) {
		$no24s = 8388608;
	}

	return $no24s;
}

sub CreateFiles {

	my @slash24s = @_;

	my $today = GetLocalTime();

	foreach $network (@slash24s) {

		my ($oc1, $oc2, $oc3, $oc4) = split /\./,$network;
		open(FILE,">$fileloc"."$oc3.$oc2.$oc1.in-addr.arpa");
		print FILE FileHeader($ns1, $ns2, $domain, $today);

		my @ClassC = PopulateC($network);
		my $counter = 0;
		for ($counter = 0; $counter < 256; $counter++) {
			$entry = @ClassC[$counter];
			my $line = "$counter\tin\tptr\t$entry.\n";
			print FILE $line;
		}
		close(FILE);
	}

	CreateForward();
}

sub PopulateC {

	my $network = shift @_;

	my ($oc1, $oc2, $oc3, $oc4) = split /\./,$network;

	my $query = $db->prepare("select label,name from networks where masklen=32 and label like '$oc1.$oc2.$oc3\%' order by network");

	$query->execute;

	my $counter = 0;
	
	my @ClassC;

	for ($counter = 0; $counter < 256; $counter++) {
		@ClassC[$counter] = "unassigned.$domain";
	}

	while ( ($label, $name) = $query->fetchrow_array ) {
		
		my ($address, $mask) = split /\//,$label;
		my ($oc1, $oc2, $oc3, $oc4) = split /\./,$address;
		@ClassC[$oc4] = $name;
	}

	return @ClassC;
}
